<!DOCTYPE kpartgui>
<kpartgui name="kmymoneyplugin-kbanking" version="5">
  <MenuBar>
    <Menu name="file">
      <Menu name="import" append="merge_import">
        <Action name="file_import_aqbanking"/>
      </Menu>
    </Menu>
    <Menu name="settings">
      <Action name="settings_aqbanking" append="configure_merge"/>
    </Menu>

  </MenuBar>
</kpartgui>
