<!DOCTYPE kpartgui>
<kpartgui name="kmymoneyplugin-ofximporter" version="2">
  <MenuBar>
    <Menu name="file">
      <Menu name="import" append="import_merge">
        <Action name="file_import_ofx" />
      </Menu>
    </Menu>
  </MenuBar>
</kpartgui>
