<!DOCTYPE kpartgui>
<kpartgui name="kmymoneyplugin-icalendarexport" version="4">
  <MenuBar>
    <Menu name="file">
      <Menu name="export" append="export_merge">
        <Action name="file_export_icalendar"/>
      </Menu>
    </Menu>
  </MenuBar>
</kpartgui>
